import 'package:dvote/dvote.dart';
import '../net/gateway.dart';
import "../gomobile/hashing.dart";

/// Returns the Poseidon hash of the given hex ECDSA public key.
/// The result is provided in base64
Future<String> digestHexClaim(String hexPublicKey) {
  return poseidonHash(hexPublicKey);
}

/// Fetch the Merkle Proof that proves that the given claim is part
/// of the Census Merkle Tree with the given Root Hash
Future<String> generateProof(String censusMerkleRootHash, String base64Claim,
    GatewayInfo gatewayInfo) async {
  if (!(censusMerkleRootHash is String) ||
      !(base64Claim is String) ||
      !(gatewayInfo is GatewayInfo)) throw Exception('Invalid parameters');
  try {
    final DVoteGateway gw =
        DVoteGateway(gatewayInfo.dvote, publicKey: gatewayInfo.publicKey);

    Map<String, dynamic> reqParams = {
      "method": "genProof",
      "censusId": censusMerkleRootHash,
      "claimData": base64Claim,
    };
    Map<String, dynamic> response = await gw.request(reqParams);
    if (!(response is Map) || !(response["siblings"] is String)) {
      throw Exception("Invalid response received from the gateway");
    }
    return response["siblings"];
  } catch (err) {
    throw Exception("The claim proof could not be generated");
  }
}
